/*****************************************************************************\
 *                                                                           *
 * Configuration.h                                                           *
 *                                                                           *
 * ------------------------------------------------------------------------- *
 * Copyright (C) 2007 Bryan Ischo <bryan@ischo.com>                          *
 *                                                                           *
 * This program is free software; you can redistribute it and/or modify it   *
 * under the terms of the GNU General Public License Version 2 as published  *
 * by the Free Software Foundation.                                          *
 *                                                                           *
 * This program is distributed in the hope that it will be useful, but       *
 * WITHOUT ANY WARRANTY; without even the implied warranty of                *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General *
 * Public License for more details.                                          *
 *                                                                           *
 * You should have received a copy of the GNU General Public License         *
 * along with this program; if not, write to:                                *
 * The Free Software Foundation, Inc.                                        *
 * 51 Franklin Street, Fifth Floor                                           *
 * Boston, MA 02110-1301, USA.                                               *
 * ------------------------------------------------------------------------- *
 *                                                                           *
 * Defines an object which reads in and parses program variables and         *
 * configuration options for xrttigen.                                       *
 *                                                                           *
\*****************************************************************************/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include <Essential/File.h>
#include <Essential/InputStream.h>
#include <Essential/OutputStream.h>
#include <Essential/RefCounted.h>
#include <Essential/RefVector.h>
#include <Essential/String.h>
#include <Essential/Vector.h>

using namespace Essential;

class Configuration
{
public:

    class Clude : public RefCounted
    {
    public:

        enum Type
        {
            TypeIn, TypeEx
        };

        enum Match
        {
            MatchExact, MatchSubOf, MatchHas
        };

        Clude(Type type, char *name);

        bool Matches(const String &name);

        Type type;
        Match match;
        String name;
        bool wildcardStartM, wildcardEndM;
    };

    Configuration(int argc, char **argv);

    virtual ~Configuration();

    bool GetRtti() const
    {
        return !disableRttiM;
    }

    String GetGeneratedName() const
    {
        return generatedNameM;
    }

    U32 GetHeaderCount() const
    {
        return vHeadersM.Count();
    }

    String GetHeader(U32 index) const
    {
        return vHeadersM[index];
    }

    InputStream &GetInputStream() const
    {
        return *pInM;
    }

    OutputStream &GetOutputStream() const
    {
        return *pOutM;
    }

    U32 GetCludeCount() const
    {
        return vCludesM.Count();
    }

    ref<Clude> GetClude(U32 index) const
    {
        return vCludesM[index];
    }
    
    bool ShouldInclude(const String &name) const;

private:

    bool disableRttiM;
    String generatedNameM;
    File *pInFileM, *pOutFileM;
    InputStream *pInM;
    OutputStream *pOutM;
    Vector<String> vHeadersM;
    RefVector<Clude> vCludesM;
};

#endif // CONFIGURATION_H
